package com.bipt.intelligentapplicationorchestrationservice.controller;

import com.bipt.intelligentapplicationorchestrationservice.config.IpConfig;
import com.bipt.intelligentapplicationorchestrationservice.entity.DeployRequest;
import com.bipt.intelligentapplicationorchestrationservice.entity.ModelSelectVO;
import com.bipt.intelligentapplicationorchestrationservice.mapper.ModelMapper;
import com.bipt.intelligentapplicationorchestrationservice.pojo.*;
import com.bipt.intelligentapplicationorchestrationservice.service.ModelDeployer;
import com.bipt.intelligentapplicationorchestrationservice.service.PublishService;
import com.bipt.intelligentapplicationorchestrationservice.util.NacosServiceUtil;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Tag(name ="服务发布相关接口")
@RestController
@RequestMapping("/publish")
@Slf4j
@CrossOrigin(origins = "http://localhost:3000")   // 生产环境指定具体域名

public class PublishController {
    @Autowired
    private PublishService publishService;

    @Autowired
    private NacosServiceUtil nacosServiceUtil;

    @Autowired
    private ModelMapper modelMapper;
    @Autowired
    private IpConfig ipConfig;

    @Autowired
    private ModelDeployer modelDeployer;

    @PostMapping
    @Operation(summary ="新增发布请求")
    @Transactional
    public OptResult<List<ServicePublishVO>> save(@RequestBody ServicePublishDTO servicePublishDTO) {
        log.info("模型发布请求:{}", servicePublishDTO);
        publishService.save(servicePublishDTO);
        //调用模型部署
        DeployRequest request = new DeployRequest();
        Long modelId = servicePublishDTO.getModelId();
        ModelVersion modelVersion = modelMapper.selectByModelId(modelId);
        String modelConfig = modelVersion.getModelConfig();
        //假设modelConfig只存GPU数据
        request.setModelId(String.valueOf(modelId));
        request.setRequiredMemory(Integer.parseInt(modelConfig));
        modelDeployer.deploy(request);
        // 获取前端传来的IP字符串
        String ipListStr = servicePublishDTO.getIp();
        if (ipListStr == null || ipListStr.trim().isEmpty()) {
            log.warn("IP列表为空，不进行Nacos注册");
            return OptResult.success();
        }
        try {
            // 使用逗号分割IP字符串
            String[] ipArray = ipListStr.split(",");
            // 循环注册每个IP到Nacos
            for (String ip : ipArray) {
                String trimmedIp = ip.trim();
                if (!trimmedIp.isEmpty()) {
                    nacosServiceUtil.registerService(
                            servicePublishDTO.getModelId().toString(),
                            trimmedIp,
                            8080,
                            servicePublishDTO.getApiUrl()
                    );
                    log.info("Nacos服务注册成功: {}", trimmedIp);
                }
            }
        } catch (Exception e) {
            log.error("Nacos服务注册失败", e);
            return OptResult.error("Nacos服务注册失败"); // 根据业务需求返回错误
        }
        return OptResult.success();
    }

    /**
     * 获取已发布的服务列表
     * @return
     */
    @GetMapping("/list")
    @Operation(summary ="获取已发布服务列表")
    public OptResult<List<ServicePublishVO>> listPublishedServices() {
        log.info("获取已发布服务列表接口被调用");
        List<ServicePublishVO> services = publishService.listPublishedServices();
        log.info("返回的数据: {}", services);
        return OptResult.success(services);
    }

    /**
     * 获取IP列表
     * @return
     */
    @GetMapping("/config/ips")
    @Operation(summary = "获取可用IP地址列表")
    public OptResult<List<String>> getAvailableIps() {
        List<String> ips = ipConfig.getIps();
        log.info("返回列表；{}",ips);
        return OptResult.success(ips);
    }
    @GetMapping("/config/ids")
    public OptResult<List<ModelSelectVO>> getModelNames(){
        List<ModelSelectVO> modelSelectVOS = publishService.getModelNames();
        log.info("获取到模型列表:{}",modelSelectVOS);
        return OptResult.success(modelSelectVOS);
    }
}
