package com.bipt.intelligentapplicationorchestrationservice.service;

import com.bipt.intelligentapplicationorchestrationservice.mapper.GpuResourceDao;
import com.bipt.intelligentapplicationorchestrationservice.exception.CacheInitException;
import com.bipt.intelligentapplicationorchestrationservice.entity.GpuResource;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.annotation.PostConstruct;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.RedisConnectionFailureException;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantLock;

@Component
public class CacheManager {
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private GpuResourceDao gpuResourceDao;

    @Autowired
    private ObjectMapper objectMapper; // 注入ObjectMapper用于类型转换

    private final ReentrantLock lock = new ReentrantLock();

    @Value("${cache.redis-key-prefix:gpu:}")
    private String keyPrefix;

    @Value("${cache.ttl-base:7200}")
    private int ttlBase;

    @Value("${cache.init-batch-size:500}")
    private int initBatchSize;

    private static final Logger log = LoggerFactory.getLogger(CacheManager.class);

    // 全量加载（带分页和分布式锁）
    @PostConstruct
    public void loadFullCache() {
        if (tryLock()) {
            try {
                int page = 0;
                while (true) {
                    List<GpuResource> batch = gpuResourceDao.findByPage(page * initBatchSize, initBatchSize);
                    if (batch.isEmpty()) break;

                    batch.forEach(this::setCacheWithTTL);
                    page++;
                }
            } finally {
                unlock();
            }
        }
    }

    // 单条缓存刷新（带版本控制）
    public void refreshCache(Long gpuId) {
        GpuResource latest = gpuResourceDao.selectByIdWithLock(gpuId);
        if (latest != null) {
            setCacheWithTTL(latest);
        }
    }

    // 批量增量同步
    public void syncCache(LocalDateTime lastSyncTime) {
        List<GpuResource> updates = gpuResourceDao.findModifiedSince(lastSyncTime);
        updates.forEach(entity -> {
            if (entity.getIsDeleted()) {
                redisTemplate.delete(buildKey(entity.getGPUId().toString()));
            } else {
                setCacheWithTTL(entity);
            }
        });
    }

    // 带随机TTL的缓存设置
    private void setCacheWithTTL(GpuResource entity) {
        String key = buildKey(entity.getGPUId().toString());

        // 直接存储实体对象，确保类型一致性
        redisTemplate.opsForValue().set(
                key,
                entity,
                ttlBase + (int)(Math.random() * 600),
                TimeUnit.SECONDS
        );
    }

    // 构建缓存键
    private String buildKey(String gpuId) {
        return keyPrefix + gpuId;
    }

    // 分布式锁操作
    private boolean tryLock() {
        try {
            return lock.tryLock(30, TimeUnit.SECONDS);
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            return false;
        }
    }

    private void unlock() {
        lock.unlock();
    }

    // 分页加载入口
    public void loadFullCache(int batchSize) {
        int page = 0;
        while (true) {
            List<GpuResource> batch = gpuResourceDao.findByPage(page * batchSize, batchSize);
            if (batch.isEmpty()) break;

            batch.forEach(this::refreshWithRetry);
            page++;
        }
    }

    // 带重试机制的缓存刷新
    public void refreshWithRetry(GpuResource entity) {
        try {
            setCacheWithTTL(entity);
        } catch (RedisConnectionFailureException ex) {
            // 3次重试逻辑
            for (int i = 0; i < 3; i++) {
                try {
                    log.info("重试第 {} 次", i + 1);
                    Thread.sleep(1000);
                    setCacheWithTTL(entity);
                    return;
                } catch (InterruptedException e) {
                    if (i == 2) {
                        throw new CacheInitException("缓存刷新失败: " + entity.getGPUId().toString());
                    }

                    log.error("重试失败", e);
                    Thread.currentThread().interrupt();
                }
            }
        }
    }

    // 获取当前缓存数量（调试用）
    public long getCacheCount() {
        return redisTemplate.keys(keyPrefix + "*").size();
    }

    public void evictCache(Long gpuId) {
        String key = buildKey(gpuId.toString());
        redisTemplate.delete(key);
    }

    // 修改获取缓存的方法，增加类型安全处理
    @SuppressWarnings("unchecked")
    public GpuResource getFromCache(String gpuId) {
        String key = buildKey(gpuId);
        Object value = redisTemplate.opsForValue().get(key);

        // 处理可能的类型不匹配问题
        if (value == null) {
            return null;
        }

        try {
            // 优先尝试直接转换
            if (value instanceof GpuResource) {
                return (GpuResource) value;
            }
            // 如果是LinkedHashMap，使用ObjectMapper转换
            else if (value instanceof java.util.LinkedHashMap) {
                return objectMapper.convertValue(value, GpuResource.class);
            }
            // 其他情况尝试序列化后反序列化（适用于JSON存储场景）
            else {
                // 先序列化为JSON字符串，再反序列化为对象
                String json = objectMapper.writeValueAsString(value);
                return objectMapper.readValue(json, GpuResource.class);
            }
        } catch (Exception e) {
            log.error("获取缓存时类型转换失败，key: {}, valueType: {}", key, value.getClass().getName(), e);
            return null;
        }
    }
}